#ifndef NETCDFCPP_MANGLE
#define NETCDFCPP_MANGLE

#define NcAtt vtknetcdfcpp_NcAtt
#define NcDim vtknetcdfcpp_NcDim
#define NcError vtknetcdfcpp_NcError
#define NcFile vtknetcdfcpp_NcFile
#define NcTypedComponent vtknetcdfcpp_NcTypedComponent
#define NcValues vtknetcdfcpp_NcValues
#define NcValues_ vtknetcdfcpp_NcValues_
#define NcValues_char vtknetcdfcpp_NcValues_char
#define NcValues_double vtknetcdfcpp_NcValues_double
#define NcValues_float vtknetcdfcpp_NcValues_float
#define NcValues_int vtknetcdfcpp_NcValues_int
#define NcValues_long vtknetcdfcpp_NcValues_long
#define NcValues_ncbyte vtknetcdfcpp_NcValues_ncbyte
#define NcValues_nclong vtknetcdfcpp_NcValues_nclong
#define NcValues_short vtknetcdfcpp_NcValues_short
#define NcVar vtknetcdfcpp_NcVar

// Rederictions for the implementation.
#define vtknetcdfcpp_NcValuesdeclare NcValuesdeclare
#define vtknetcdfcpp_NcValuesimplement NcValuesimplement

#endif
